@@ -1,10 +1,13 @@
-/********************************************************
-/                   MANDELBULBER                        *
-/                                                       *
-/ author: Krzysztof Marczak                             *
-/ contact: buddhi1980@gmail.com                         *
-/ licence: GNU GPL                                      *
-********************************************************/
+/*********************************************************
+ /                   MANDELBULBER
+ / some functions for different calculations
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
 
 /*
  * common_math.cpp
@@ -52,37 +55,7 @@ double dMin(double a, double b, double c)
 	return c;
 }
 
-void QuickSortZBuffer(sSortZ *dane, int l, int p)
-{
-  int i,j; //robocze indeksy
-  sSortZ x,y; //robocze zmienne typu sortowany obiekt
 
-  i=l;           //{ i : = minimalny klucz }
-  j=p;           //{ j : = maksymalny klucz }
-  x=dane[(l+p)/2];  //{ x - wyraz srodkowy sortowanej tablicy }
-  do
-  {             //{ cala historie bedziemy powtarzac tak dlugo jak . . . }
-	while(dane[i].z < x.z)              //{ dopoki jestesmy na lewo od srodkowego }
-	{
-	  i++;                       //{ ...powiekszamy indeks i }
-	}
-	while(x.z < dane[j].z)                  //{ dopoki na prawo od srodkowego }
-	{
-	  j--;                       //{ ...zmniejszamy indeks j }
-	}
-	if(i<=j)                        //{ jezeli i <= j wtedy : }
-	{                               //{ zamieniamy miejscami wyrazy i, j }
-	  y=dane[i];                      //{ podstawienie do zmiennej roboczej }
-	  dane[i]=dane[j];                    //{ pierwszy etap zamiany }
-	  dane[j]=y;                      //{ drugi etap zamiany }
-	  i++;                       //{ zwiekszenie indeksu i }
-	  j--;                       //{ zmniejszenie indeksu j }
-	}
-  }
-  while(i<=j);      //{ nie zostanie spelniony warunek i>j }
-  if(l<j) QuickSortZBuffer(dane, l, j);
-  if(i<p) QuickSortZBuffer(dane, i, p);
-}
 
 CVector3 Projection3D(CVector3 point, CVector3 vp, CRotationMatrix mRot, enumPerspectiveType perspectiveType, double fov, double zoom)
 {
@@ -130,3 +103,48 @@ CVector3 Projection3D(CVector3 point, CVector3 vp, CRotationMatrix mRot, enumPer
 	return result;
 }
 
+CVector3 InvProjection3D(CVector3 point, CVector3 vp, CRotationMatrix mRotInv, enumPerspectiveType perspectiveType, double fov, double zoom, double imgWidth, double imgHeight)
+{
+	CVector3 screenPoint;
+	CVector3 baseZ(0.0, 1.0, 0.0);
+
+	double aspectRatio = (double) imgWidth / imgHeight;
+
+	CVector3 start;
+	if (perspectiveType == fishEye || perspectiveType == equirectangular)
+	{
+		start = vp;
+	}
+	else
+	{
+		start = vp - baseZ * (1.0 / fov * zoom);
+	}
+	CVector3 viewVector = point - start;
+	viewVector = mRotInv.RotateVector(viewVector);
+
+	double x, y, z;
+	if (perspectiveType == fishEye)
+	{
+		z = viewVector.Length();
+		if(viewVector.y < 0) z = -z;
+		viewVector.Normalize();
+		double r = sqrt(viewVector.x * viewVector.x + viewVector.z * viewVector.z);
+		double r2 = asin(r) / (M_PI * 0.5);
+		x = (viewVector.x / fov) * r2 / r / 2.0;
+		y = (viewVector.z / fov) * r2 / r / 2.0;
+	}
+	else
+	{
+
+		viewVector.x /= viewVector.y;
+		viewVector.z /= viewVector.y;
+		x = viewVector.x / fov;
+		y = viewVector.z / fov;
+		z = viewVector.y;
+	}
+	screenPoint.x = (x / aspectRatio + 0.5) * imgWidth;
+	screenPoint.y = (y + 0.5) * imgHeight;
+	screenPoint.z = z;
+
+	return screenPoint;
+}
