@@ -1,18 +1,14 @@
-/********************************************************
- /                   MANDELBULBER                        *
- /                                                       *
- / author: Krzysztof Marczak                             *
- / contact: buddhi1980@gmail.com                         *
- / licence: GNU GPL                                      *
+/*********************************************************
+ /                   MANDELBULBER
+ / functions for operating on files, image loading and
+ / saving
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
  ********************************************************/
 
-/*
- * files.cpp
- *
- *  Created on: 2010-01-23
- *      Author: krzysztof
- */
-
 #include <cstdio>
 #include <string.h>
 #define PNG_DEBUG 3
@@ -194,25 +190,35 @@ void SavePNG(const char *filename, int /*quality*/, int width, int height, png_b
 {
 	/* create file */
 	FILE *fp = fopen(filename, "wb");
-	if (!fp) fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
-
+	if (!fp)
+	{
+		fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
+		return;
+	}
 	/* initialize stuff */
 	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 
-	if (!png_ptr) fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+	if (!png_ptr)
+	{
+		fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+		fclose(fp);
+		return;
+	}
 
 	png_info *info_ptr = png_create_info_struct(png_ptr);
 	if (!info_ptr)
 	{
 		fprintf(stderr, "[write_png_file] png_create_info_struct failed");
+		fclose(fp);
 		return;
 	}
 
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during init_io");
+		fclose(fp);
 		return;
-	};
+	}
 
 	png_init_io(png_ptr, fp);
 
@@ -220,6 +226,7 @@ void SavePNG(const char *filename, int /*quality*/, int width, int height, png_b
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during writing header");
+		fclose(fp);
 		return;
 	}
 
@@ -231,6 +238,7 @@ void SavePNG(const char *filename, int /*quality*/, int width, int height, png_b
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during writing bytes");
+		fclose(fp);
 		return;
 	}
 
@@ -246,6 +254,8 @@ void SavePNG(const char *filename, int /*quality*/, int width, int height, png_b
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during end of write");
+		delete[] row_pointers;
+		fclose(fp);
 		return;
 	}
 
@@ -259,124 +269,39 @@ void SavePNG(const char *filename, int /*quality*/, int width, int height, png_b
 void SaveFromTilesPNG16(const char *filename, int width, int height, int tiles)
 {
 	/* create file */
-		string filenamePNG(filename);
-		filenamePNG += "_fromTiles.png";
-		FILE *fp = fopen(filenamePNG.c_str(), "wb");
-		if (!fp) fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
-
-		/* initialize stuff */
-		png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-
-		if (!png_ptr) fprintf(stderr, "[write_png_file] png_create_write_struct failed");
-
-		png_info *info_ptr = png_create_info_struct(png_ptr);
-		if (!info_ptr)
-		{
-			fprintf(stderr, "[write_png_file] png_create_info_struct failed");
-			return;
-		}
-
-		if (setjmp(png_jmpbuf(png_ptr)))
-		{
-			fprintf(stderr, "[write_png_file] Error during init_io");
-			return;
-		};
-
-		png_init_io(png_ptr, fp);
-
-		/* write header */
-		if (setjmp(png_jmpbuf(png_ptr)))
-		{
-			fprintf(stderr, "[write_png_file] Error during writing header");
-			return;
-		}
-
-		png_set_IHDR(png_ptr, info_ptr, width * tiles, height * tiles, 16, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
-
-		png_write_info(png_ptr, info_ptr);
-		png_set_swap(png_ptr);
-
-		/* write bytes */
-		if (setjmp(png_jmpbuf(png_ptr)))
-		{
-			fprintf(stderr, "[write_png_file] Error during writing bytes");
-			return;
-		}
-
-		FILE **files = new FILE*[tiles];
-
-		sRGB16 *rowBuffer = new sRGB16[width*tiles];
-
-		for(int tileRow = 0; tileRow<tiles; tileRow++)
-		{
-			printf("Compiling image from tiles, row %d\n", tileRow);
-			for(int tile = 0; tile<tiles; tile++)
-			{
-				int fileNumber = tile + tileRow * tiles;
-				string filename2 = IndexFilename(filename, "tile", fileNumber);
-				files[tile] = fopen(filename2.c_str(), "rb");
-			}
-
-			for(int y = 0; y < height; y++)
-			{
-				for(int tile = 0; tile<tiles; tile++)
-				{
-					size_t result = fread(&rowBuffer[tile*width], 1, sizeof(sRGB16)*width, files[tile]);
-					if (result != sizeof(sRGB16)*width)
-					{
-						printf("Reading error of image tile files");
-						return;
-					}
-				}
-				png_write_rows(png_ptr, (png_bytep*)&rowBuffer,	1);
-			}
-
-			for(int tile = 0; tile<tiles; tile++)
-			{
-				fclose(files[tile]);
-				int fileNumber = tile + tileRow * tiles;
-				string filename2 = IndexFilename(filename, "tile", fileNumber);
-				remove(filename2.c_str());
-			}
-		}
-
-		/* end write */
-		if (setjmp(png_jmpbuf(png_ptr)))
-		{
-			fprintf(stderr, "[write_png_file] Error during end of write");
-			return;
-		}
-
-		png_write_end(png_ptr, NULL);
-
-		delete[] rowBuffer;
-
-		fclose(fp);
-}
-
-void SavePNG16(const char *filename, int /*quality*/, int width, int height, sRGB16* image16)
-{
-	/* create file */
-	FILE *fp = fopen(filename, "wb");
-	if (!fp) fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
+	string filenamePNG(filename);
+	filenamePNG += "_fromTiles.png";
+	FILE *fp = fopen(filenamePNG.c_str(), "wb");
+	if (!fp)
+	{
+		fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
+		return;
+	}
 
 	/* initialize stuff */
 	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 
-	if (!png_ptr) fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+	if (!png_ptr)
+	{
+		fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+		fclose(fp);
+		return;
+	}
 
 	png_info *info_ptr = png_create_info_struct(png_ptr);
 	if (!info_ptr)
 	{
 		fprintf(stderr, "[write_png_file] png_create_info_struct failed");
+		fclose(fp);
 		return;
 	}
 
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during init_io");
+		fclose(fp);
 		return;
-	};
+	}
 
 	png_init_io(png_ptr, fp);
 
@@ -384,6 +309,121 @@ void SavePNG16(const char *filename, int /*quality*/, int width, int height, sRG
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during writing header");
+		fclose(fp);
+		return;
+	}
+
+	png_set_IHDR(png_ptr, info_ptr, width * tiles, height * tiles, 16, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
+
+	png_write_info(png_ptr, info_ptr);
+	png_set_swap(png_ptr);
+
+	/* write bytes */
+	if (setjmp(png_jmpbuf(png_ptr)))
+	{
+		fprintf(stderr, "[write_png_file] Error during writing bytes");
+		fclose(fp);
+		return;
+	}
+
+	FILE **files = new FILE*[tiles];
+
+	sRGB16 *rowBuffer = new sRGB16[width * tiles];
+
+	for (int tileRow = 0; tileRow < tiles; tileRow++)
+	{
+		printf("Compiling image from tiles, row %d\n", tileRow);
+		for (int tile = 0; tile < tiles; tile++)
+		{
+			int fileNumber = tile + tileRow * tiles;
+			string filename2 = IndexFilename(filename, "tile", fileNumber);
+			files[tile] = fopen(filename2.c_str(), "rb");
+		}
+
+		for (int y = 0; y < height; y++)
+		{
+			for (int tile = 0; tile < tiles; tile++)
+			{
+				size_t result = fread(&rowBuffer[tile * width], 1, sizeof(sRGB16) * width, files[tile]);
+				if (result != sizeof(sRGB16) * width)
+				{
+					printf("Reading error of image tile files");
+					for (int tile2 = 0; tile2 < tiles; tile2++)
+						fclose(files[tile2]);
+					return;
+				}
+			}
+			png_write_rows(png_ptr, (png_bytep*) &rowBuffer, 1);
+		}
+
+		for (int tile = 0; tile < tiles; tile++)
+		{
+			fclose(files[tile]);
+			int fileNumber = tile + tileRow * tiles;
+			string filename2 = IndexFilename(filename, "tile", fileNumber);
+			remove(filename2.c_str());
+		}
+	}
+
+	/* end write */
+	if (setjmp(png_jmpbuf(png_ptr)))
+	{
+		fprintf(stderr, "[write_png_file] Error during end of write");
+		delete[] rowBuffer;
+		fclose(fp);
+		return;
+	}
+
+	png_write_end(png_ptr, NULL);
+
+	delete[] rowBuffer;
+	delete[] files;
+
+	fclose(fp);
+}
+
+void SavePNG16(const char *filename, int /*quality*/, int width, int height, sRGB16* image16)
+{
+	/* create file */
+	FILE *fp = fopen(filename, "wb");
+	if (!fp)
+	{
+	 fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
+	  return;
+	}
+
+	/* initialize stuff */
+	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+
+	if (!png_ptr)
+	{
+		fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+		fclose(fp);
+		return;
+	}
+
+	png_info *info_ptr = png_create_info_struct(png_ptr);
+	if (!info_ptr)
+	{
+		fprintf(stderr, "[write_png_file] png_create_info_struct failed");
+		fclose(fp);
+		return;
+	}
+
+	if (setjmp(png_jmpbuf(png_ptr)))
+	{
+		fprintf(stderr, "[write_png_file] Error during init_io");
+		fclose(fp);
+		return;
+	}
+
+	png_init_io(png_ptr, fp);
+
+	/* write header */
+	if (setjmp(png_jmpbuf(png_ptr)))
+	{
+		fprintf(stderr, "[write_png_file] Error during writing header");
+		fclose(fp);
 		return;
 	}
 
@@ -396,6 +436,7 @@ void SavePNG16(const char *filename, int /*quality*/, int width, int height, sRG
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during writing bytes");
+		fclose(fp);
 		return;
 	}
 
@@ -411,6 +452,8 @@ void SavePNG16(const char *filename, int /*quality*/, int width, int height, sRG
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during end of write");
+		delete [] row_pointers;
+		fclose(fp);
 		return;
 	}
 
@@ -425,25 +468,35 @@ void SavePNG16Alpha(const char *filename, int /*quality*/, int width, int height
 {
 	/* create file */
 	FILE *fp = fopen(filename, "wb");
-	if (!fp) fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
-
+	if (!fp)
+	{
+	   	fprintf(stderr, "[write_png_file] File %s could not be opened for writing", filename);
+		return;
+	}
 	/* initialize stuff */
 	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 
-	if (!png_ptr) fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+	if (!png_ptr)
+	{ 
+		fprintf(stderr, "[write_png_file] png_create_write_struct failed");
+		fclose(fp);
+		return;
+	}
 
 	png_info *info_ptr = png_create_info_struct(png_ptr);
 	if (!info_ptr)
 	{
 		fprintf(stderr, "[write_png_file] png_create_info_struct failed");
+		fclose(fp);
 		return;
 	}
 
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during init_io");
+		fclose(fp);
 		return;
-	};
+	}
 
 	png_init_io(png_ptr, fp);
 
@@ -451,6 +504,7 @@ void SavePNG16Alpha(const char *filename, int /*quality*/, int width, int height
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during writing header");
+		fclose(fp);
 		return;
 	}
 
@@ -463,6 +517,7 @@ void SavePNG16Alpha(const char *filename, int /*quality*/, int width, int height
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during writing bytes");
+		fclose(fp);
 		return;
 	}
 
@@ -474,7 +529,7 @@ void SavePNG16Alpha(const char *filename, int /*quality*/, int width, int height
 		for (int x = 0; x < width; x++)
 		{
 			unsigned long int ptr = x+y*width;
-			sRGB16 pixel = image->GetPixelImage(x,y);
+			sRGB16 pixel = image->GetPixelImage16(x,y);
 			image16[ptr].R = pixel.R;
 			image16[ptr].G = pixel.G;
 			image16[ptr].B = pixel.B;
@@ -493,6 +548,9 @@ void SavePNG16Alpha(const char *filename, int /*quality*/, int width, int height
 	if (setjmp(png_jmpbuf(png_ptr)))
 	{
 		fprintf(stderr, "[write_png_file] Error during end of write");
+		delete[] row_pointers;
+		delete[] image16;
+		fclose(fp);
 		return;
 	}
 
@@ -559,6 +617,8 @@ int fcopy(const char *source, const char *dest)
 	if (result != lSize)
 	{
 		printf("Can't read source file for copying: %s\n", source);
+		delete[] buffer;
+		fclose(pFile);
 		return 2;
 	}
 	fclose(pFile);
@@ -569,6 +629,7 @@ int fcopy(const char *source, const char *dest)
 	if (pFile == NULL)
 	{
 		printf("Can't open destination file for copying: %s\n", dest);
+		delete[] buffer;
 		return 3;
 	}
 	fwrite(buffer, 1, lSize, pFile);
@@ -602,6 +663,7 @@ void BufferNormalize16(sRGB16 *buffer, unsigned int size)
 	}
 }
 
+/*
 void SaveAllImageLayers(const char *filename, cImage *image)
 {
 	unsigned int width = image->GetWidth();
@@ -680,7 +742,7 @@ void SaveAllImageLayers(const char *filename, cImage *image)
 	SavePNG16((file+"_alpha.png").c_str(), 100, width, height, buffer16);
 
 	//normalize zBuffer
-	float *zbuffer = image->GetZBufferPtr();
+	double *zbuffer = image->GetZBufferPtr();
 	float minZ = 1.0e50;
 	float maxZ = 0.0;
 	for (unsigned int i = 0; i < size; i++)
@@ -703,6 +765,8 @@ void SaveAllImageLayers(const char *filename, cImage *image)
 
 	delete [] buffer16;
 }
+*/
+
 
 string removeFileExtension(const string &filename)
 {
