@@ -1,32 +1,19 @@
-/********************************************************
- /                   MANDELBULBER                        *
- /                                                       *
- / author: Krzysztof Marczak                             *
- / contact: buddhi1980@gmail.com                         *
- / licence: GNU GPL                                      *
+/*********************************************************
+ /                   MANDELBULBER
+ / fractal iteration functions and distance estimation
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
  ********************************************************/
 
-/*
- * fractal.cpp
- *
- *  Created on: 2010-01-23
- *      Author: krzysztof
- */
-
 #include "Render3D.h"
 #include "interface.h"
 #include "primitives.h"
 #include <stdlib.h>
 
-/**
- * Compute the fractal at the point, in one of the various modes
- *
- * Mode: normal: Returns distance
- *		 fake_ao: Returns minimum radius
- *		 colouring: Returns colour index
- *		 delta_DE1, delta_DE2: Returns radius
- */
-
 unsigned int MixNumbers(double a, double b, double c)
 {
 	unsigned int result = int(a*12356312.0)^int(b*23564234.0)^int(c*35564353.0);
@@ -41,6 +28,14 @@ int Noise(int seed)
 	return abs((x * (x * x * 15731 + 789221) + 1376312589) & 0x7fffffff);
 }
 
+/**
+ * Compute the fractal at the point, in one of the various modes
+ *
+ * Mode: normal: Returns distance
+ *		 fake_ao: Returns minimum radius
+ *		 colouring: Returns colour index
+ *		 delta_DE1, delta_DE2: Returns radius
+ */
 
 template<int Mode>
 double Compute(CVector3 z, const sFractal &par, int *iter_count)
@@ -68,7 +63,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 	double tglad_factor1 = fR2 / mR2;
 
 	double tgladDE = 1.0;
-
+	double orbitTrapTotal = 0.0;
 	double scale = par.mandelbox.doubles.scale;
 
 	enumFractalFormula actualFormula = par.formula;
@@ -1107,6 +1102,8 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 				break;
 			case none:
 				break;
+			case ocl_custom:
+				break;
 		}
 
 		//************************** iteration terminate conditions *****************
@@ -1125,10 +1122,10 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 		{
 			CVector3 delta = z - par.doubles.fakeLightsOrbitTrap;
 			distance = delta.Length();
-			if (L >= par.fakeLightsMinIter && L <= par.fakeLightsMaxIter && distance < min) min = distance;
+			if (L >= par.fakeLightsMinIter && L <= par.fakeLightsMaxIter) orbitTrapTotal += (1.0f/(distance*distance));
 			if (distance > 1000)
 			{
-				distance = min;
+				distance = orbitTrapTotal;
 				break;
 			}
 		}
@@ -1175,7 +1172,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 					distance = z.Length();
 					if (distance > 1e15)
 					{
-						distance = (L - log(log(r) / log(N)) / log(p))/100.0;
+						distance = (L - log(log(r) / log((double)N)) / log(p))/100.0;
 						break;
 					}
 				}
@@ -1249,54 +1246,22 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 {
 	int L;
 	double distance;
-	params.specialColour = 0;
+	params.objectOut = objFractal;
+
+	double limitBoxDist = 0.0;
 
 	if (params.limits_enabled)
 	{
-		bool limit = false;
-		double distance_a = 0;
-		double distance_b = 0;
-		double distance_c = 0;
+		double distance_a = MAX(point.x - params.doubles.amax, -(point.x - params.doubles.amin));
+		double distance_b = MAX(point.y - params.doubles.bmax, -(point.y - params.doubles.bmin));
+		double distance_c = MAX(point.z - params.doubles.cmax, -(point.z - params.doubles.cmin));
+		limitBoxDist = dMax(distance_a, distance_b, distance_c);
 
-		if (point.x < params.doubles.amin)
-		{
-			distance_a = fabs(params.doubles.amin - point.x) + params.doubles.detailSize;
-			limit = true;
-		}
-		if (point.x > params.doubles.amax)
-		{
-			distance_a = fabs(params.doubles.amax - point.x) + params.doubles.detailSize;
-			limit = true;
-		}
-
-		if (point.y < params.doubles.bmin)
-		{
-			distance_a = fabs(params.doubles.bmin - point.y) + params.doubles.detailSize;
-			limit = true;
-		}
-		if (point.y > params.doubles.bmax)
-		{
-			distance_b = fabs(params.doubles.bmax - point.y) + params.doubles.detailSize;
-			limit = true;
-		}
-
-		if (point.z < params.doubles.cmin)
-		{
-			distance_c = fabs(params.doubles.cmin - point.z) + params.doubles.detailSize;
-			limit = true;
-		}
-		if (point.z > params.doubles.cmax)
-		{
-			distance_c = fabs(params.doubles.cmax - point.z) + params.doubles.detailSize;
-			limit = true;
-		}
-
-		if (limit)
+		if(limitBoxDist > params.doubles.detailSize)
 		{
 			if (max_iter != NULL)
-				*max_iter = false;
-			distance = dMax(distance_a, distance_b, distance_c);
-			return distance;
+					*max_iter = false;
+			return limitBoxDist;
 		}
 	}
 
@@ -1311,10 +1276,11 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 				else *max_iter = false;
 			}
 			params.itersOut = L;
+			if (distance < 0) distance = 0;
 
 			if (L < params.minN && distance < params.doubles.detailSize) distance = params.doubles.detailSize;
 
-			if (params.interiorMode)
+			if (params.interiorMode && !params.normalCalculationMode)
 			{
 				if (distance < 0.5 * params.doubles.detailSize || L == (int)params.doubles.N)
 				{
@@ -1322,6 +1288,22 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 					if (max_iter != NULL) *max_iter = false;
 				}
 			}
+			else if(params.interiorMode && params.normalCalculationMode)
+			{
+				if (distance < 0.9 * params.doubles.detailSize)
+				{
+					distance = params.doubles.detailSize - distance;
+					if (max_iter != NULL) *max_iter = false;
+				}
+			}
+
+			if (params.iterThresh && L < (int)params.doubles.N)
+			{
+				if(distance < params.doubles.detailSize)
+				{
+					distance = params.doubles.detailSize * 1.01;
+				}
+			}
 		}
 		else
 		{
@@ -1360,6 +1342,8 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 				distance = 0.5 * r * log(r) / dr;
 			}
 
+			if (distance < 0) distance = 0;
+
 			if (retval == (int)params.doubles.N)
 			{
 				if (max_iter != NULL) *max_iter = true;
@@ -1369,14 +1353,31 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 
 			if (L < params.minN && distance < params.doubles.detailSize) distance = params.doubles.detailSize;
 
-			if (params.interiorMode)
+			if (params.interiorMode && !params.normalCalculationMode)
 			{
-				if (distance < 0.5 * params.doubles.detailSize || retval == 256)
+				if (distance < 0.5 * params.doubles.detailSize || L == (int)params.doubles.N)
 				{
 					distance = params.doubles.detailSize;
 					if (max_iter != NULL) *max_iter = false;
 				}
 			}
+			else if(params.interiorMode && params.normalCalculationMode)
+			{
+				if (distance < 0.9 * params.doubles.detailSize)
+				{
+					distance = params.doubles.detailSize - distance;
+					if (max_iter != NULL) *max_iter = false;
+				}
+			}
+
+			if (params.iterThresh && retval < (int)params.doubles.N)
+			{
+				if(distance < params.doubles.detailSize)
+				{
+					distance = params.doubles.detailSize * 1.01;
+				}
+			}
+
 		}
 	}
 	else
@@ -1389,7 +1390,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.planeEnable)
 	{
 		double planeDistance = PrimitivePlane(point, params.doubles.primitives.planeCentre, params.doubles.primitives.planeNormal);
-		if(!params.primitives.onlyPlane && planeDistance < distance) 	params.specialColour = 253;
+		if(!params.primitives.onlyPlane && planeDistance < distance) 	params.objectOut = objPlane;
 		distance = (planeDistance < distance) ? planeDistance : distance;
 
 	}
@@ -1398,7 +1399,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.boxEnable)
 	{
 		double boxDistance = PrimitiveBox(point, params.doubles.primitives.boxCentre, params.doubles.primitives.boxSize);
-		if(boxDistance < distance) 	params.specialColour = 252;
+		if(boxDistance < distance) 	params.objectOut = objBox;
 		distance = (boxDistance < distance) ? boxDistance : distance;
 	}
 
@@ -1406,7 +1407,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.invertedBoxEnable)
 	{
 		double boxDistance = PrimitiveInvertedBox(point, params.doubles.primitives.invertedBoxCentre, params.doubles.primitives.invertedBoxSize);
-		if(boxDistance < distance) 	params.specialColour = 251;
+		if(boxDistance < distance) 	params.objectOut = objBoxInv;
 		distance = (boxDistance < distance) ? boxDistance : distance;
 	}
 
@@ -1414,7 +1415,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.sphereEnable)
 	{
 		double sphereDistance = PrimitiveSphere(point, params.doubles.primitives.sphereCentre, params.doubles.primitives.sphereRadius);
-		if(sphereDistance < distance) 	params.specialColour = 250;
+		if(sphereDistance < distance) 	params.objectOut = objSphere;
 		distance = (sphereDistance < distance) ? sphereDistance : distance;
 	}
 
@@ -1422,7 +1423,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.invertedSphereEnable)
 	{
 		double sphereDistance = PrimitiveInvertedSphere(point, params.doubles.primitives.invertedSphereCentre, params.doubles.primitives.invertedSphereRadius);
-		if(sphereDistance < distance) 	params.specialColour = 249;
+		if(sphereDistance < distance) 	params.objectOut = objSphereInv;
 		distance = (sphereDistance < distance) ? sphereDistance : distance;
 	}
 
@@ -1430,13 +1431,19 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.waterEnable)
 	{
 		double waterDistance = PrimitiveWater(point, params.doubles.primitives.waterHeight, params.doubles.primitives.waterAmplitude,
-				params.doubles.primitives.waterLength, params.doubles.primitives.waterRotation, params.primitives.waterIterations, 0.1, params.frameNo);
-		if(waterDistance < distance) 	params.specialColour = 248;
+				params.doubles.primitives.waterLength, params.doubles.primitives.waterRotation, params.primitives.waterIterations, params.doubles.primitives.waterAnimSpeed, params.frameNo);
+		if(waterDistance < distance) 	params.objectOut = objWater;
 		distance = (waterDistance < distance) ? waterDistance : distance;
 	}
 
-	if (distance < 0) distance = 0;
 
+	if (params.limits_enabled)
+	{
+		if (limitBoxDist < params.doubles.detailSize)
+		{
+			distance = MAX(distance, limitBoxDist);
+		}
+	}
 	return distance;
 }
 
