@@ -1,23 +1,21 @@
-/********************************************************
- /                   MANDELBULBER                        *
- /                                                       *
- / author: Krzysztof Marczak                             *
- / contact: buddhi1980@gmail.com                         *
- / licence: GNU GPL                                      *
+/*********************************************************
+ /                   MANDELBULBER
+ / functions for image post-processing
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
  ********************************************************/
 
-/*
- * image.cpp
- *
- *  Created on: 2010-01-23
- *      Author: krzysztof
- */
 
 #include <string.h>
 
 #include "image.h"
 #include "Render3D.h"
 #include "interface.h"
+#include "shaders.h"
 
 sRGB *buddhabrotImg;
 bool isBuddhabrot = false;
@@ -56,7 +54,7 @@ void NewPalette(sRGB *p, double saturation)
 
 }
 
-void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
+void PostRendering_DOF(cImage *image, double deep, double neutral)
 {
 	isPostRendering = true;
 
@@ -65,13 +63,13 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 
 	sRGB16 *temp_image = new sRGB16[width * height];
 	unsigned short *temp_alpha = new unsigned short[width * height];
-	sSortZ *temp_sort = new sSortZ[width * height];
+	sSortZ<float> *temp_sort = new sSortZ<float>[width * height];
 	for (int y = 0; y < height; y++)
 	{
 		for (int x = 0; x < width; x++)
 		{
 			int ptr = x + y * width;
-			temp_image[ptr] = image->GetPixelImage(x, y);
+			temp_image[ptr] = image->GetPixelImage16(x, y);
 			temp_alpha[ptr] = image->GetPixelAlpha(x, y);
 			temp_sort[ptr].z = image->GetPixelZBuffer(x, y);
 			temp_sort[ptr].i = ptr;
@@ -94,16 +92,14 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 			gtk_main_iteration();
 	}
 
-	double min = -1.0 / persp;
-
 	//Randomize Z-buffer
 	int imgSize = height*width;
 	for(int i=imgSize-1; i>=0; i--)
 	{
-		sSortZ temp;
+		sSortZ<float> temp;
 		temp = temp_sort[i];
 		double z1 = temp.z;
-		double size1 = (z1 - neutral) / (z1 - min) * deep;
+		double size1 = (z1 - neutral) / z1 * deep;
 
 		int randomStep = i;
 
@@ -113,9 +109,9 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 		{
 			ii = i - Random(randomStep);
 			if (ii <= 0) ii = 0;
-			sSortZ temp2 = temp_sort[ii];
+			sSortZ<float> temp2 = temp_sort[ii];
 			double z2 = temp2.z;
-			double size2 = (z2 - neutral) / (z2 - min) * deep;
+			double size2 = (z2 - neutral) / z2 * deep;
 
 			if(size1 == 0 && size2 ==0) done = true;
 
@@ -169,7 +165,7 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 		int x = ii % width;
 		int y = ii / width;
 		double z = image->GetPixelZBuffer(x, y);
-		double blur = fabs((double) z - neutral) / (z - min) * deep;
+		double blur = fabs((double) z - neutral) / z * deep;
 		if (blur > 100) blur = 100.0;
 		int size = blur;
 		sRGB16 center = temp_image[x + y * width];
@@ -191,14 +187,14 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 					if (op > 0.0)
 					{
 						double opN = 1.0 - op;
-						sRGB16 old = image->GetPixelImage(xx, yy);
+						sRGB16 old = image->GetPixelImage16(xx, yy);
 						unsigned short old_alpha = image->GetPixelAlpha(xx, yy);
 						sRGB16 pixel;
 						pixel.R = old.R * opN + center.R * op;
 						pixel.G = old.G * opN + center.G * op;
 						pixel.B = old.B * opN + center.B * op;
 						unsigned short alpha = old_alpha * opN + center_alpha * op;
-						image->PutPixelImage(xx, yy, pixel);
+						image->PutPixelImage16(xx, yy, pixel);
 						image->PutPixelAlpha(xx, yy, alpha);
 					}
 				}
@@ -215,6 +211,7 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 
 			if(refreshCount > 600)
 			{
+				//image->CompileImage();
 				image->ConvertTo8bit();
 				image->UpdatePreview();
 				image->RedrawInWidget(renderWindow.drawingArea);
@@ -251,6 +248,7 @@ void PostRendering_DOF(cImage *image, double deep, double neutral, double persp)
 
 void ThreadSSAO(void *ptr)
 {
+
 	sSSAOparams *param;
 	param = (sSSAOparams*) ptr;
 
@@ -277,12 +275,17 @@ void ThreadSSAO(void *ptr)
 	enumPerspectiveType perspectiveType = param->perspectiveType;
 	double fov = param->persp;
 
+	sImageAdjustments *imageAdjustments = image->GetImageAdjustments();
+	double intensity = imageAdjustments->globalIlum;
+
 	for (int y = threadNo * progressive; y < height; y += NR_THREADS * progressive)
 	{
 
 		for (int x = 0; x < width; x += progressive)
 		{
 			double z = image->GetPixelZBuffer(x, y);
+			unsigned short opacity16 = image->GetPixelOpacity(x, y);
+			double opacity = opacity16 / 65535.0;
 			double total_ambient = 0;
 
 			if (z < 1e19)
@@ -293,12 +296,16 @@ void ThreadSSAO(void *ptr)
 				{
 					x2 = M_PI * ((double) x / width - 0.5) * aspectRatio;
 					y2 = M_PI * ((double) y / height - 0.5);
-					x2 = sin(fov * x2) * z;
-					y2 = sin(fov * y2) * z;
+					double r = sqrt(x2 * x2 + y2 * y2);
+					if(r != 0.0)
+					{
+						x2 = x2 / r * sin(r * fov) * z;
+						y2 = y2 / r * sin(r * fov) * z;
+					}
 				}
 				else if(perspectiveType == equirectangular)
 				{
-					x2 = M_PI * ((double) x / width - 0.5) * aspectRatio;
+					x2 = M_PI * ((double) x / width - 0.5) * aspectRatio; //--------- do sprawdzenia
 					y2 = M_PI * ((double) y / height - 0.5);
 					x2 = sin(fov * x2) * cos(fov * y2) * z;
 					y2 = sin(fov * y2) * z;
@@ -307,8 +314,8 @@ void ThreadSSAO(void *ptr)
 				{
 					x2 = ((double) x / width - 0.5) * aspectRatio;
 					y2 = ((double) y / height - 0.5);
-					x2 = x2 * (1.0 + z * persp);
-					y2 = y2 * (1.0 + z * persp);
+					x2 = x2 * z * persp;
+					y2 = y2 * z * persp;
 				}
 
 				double ambient = 0;
@@ -333,14 +340,18 @@ void ThreadSSAO(void *ptr)
 						double xx2, yy2;
 						if (perspectiveType == fishEye)
 						{
-							xx2 = M_PI * (xx / width - 0.5) * aspectRatio;
-							yy2 = M_PI * (yy / height - 0.5);
-							xx2 = sin(fov * xx2) * z2;
-							yy2 = sin(fov * yy2) * z2;
+							xx2 = M_PI * ((double) xx / width - 0.5) * aspectRatio;
+							yy2 = M_PI * ((double) yy / height - 0.5);
+							double r2 = sqrt(xx2 * xx2 + yy2 * yy2);
+							if(r != 0.0)
+							{
+								xx2 = xx2 / r2 * sin(r2 * fov) * z2;
+								yy2 = yy2 / r2 * sin(r2 * fov) * z2;
+							}
 						}
 						else if (perspectiveType == equirectangular)
 						{
-							xx2 = M_PI * (xx / width - 0.5) * aspectRatio;
+							xx2 = M_PI * (xx / width - 0.5) * aspectRatio; //--------- do sprawdzenia
 							yy2 = M_PI * (yy / height - 0.5);
 							xx2 = sin(fov * xx2) * cos(fov * yy2) * z2;
 							yy2 = sin(fov * yy2) * z2;
@@ -349,8 +360,8 @@ void ThreadSSAO(void *ptr)
 						{
 							xx2 = (xx / width - 0.5) * aspectRatio;
 							yy2 = (yy / height - 0.5);
-							xx2 = xx2 * (1.0 + z2 * persp);
-							yy2 = yy2 * (1.0 + z2 * persp);
+							xx2 = xx2 * (z2 * persp);
+							yy2 = yy2 * (z2 * persp);
 						}
 
 						double dx = xx2 - x2;
@@ -373,32 +384,26 @@ void ThreadSSAO(void *ptr)
 
 			}
 
-			sRGB16 ambient = { total_ambient * 4096.0, total_ambient * 4096.0, total_ambient * 4096.0 };
-			if (!image->IsLowMemMode()) image->PutPixelAmbient(x, y, ambient);
-			else
+			sRGB8 colour = image->GetPixelColor(x, y);
+			sRGB16 pixel = image->GetPixelImage16(x, y);
+			double R = pixel.R + 65535.0 * colour.R / 256.0 * total_ambient * intensity * (1.0 - opacity);
+			double G = pixel.G + 65535.0 * colour.G / 256.0 * total_ambient * intensity * (1.0 - opacity);
+			double B = pixel.B + 65535.0 * colour.B / 256.0 * total_ambient * intensity * (1.0 - opacity);
+			if (R > 65535) R = 65535;
+			if (G > 65535) G = 65535;
+			if (B > 65535) B = 65535;
+			for(int sqY =0; sqY<progressive; sqY++)
 			{
-				unsigned short colorIndex = image->GetPixelColor(x,y);
-				sRGB16 oldPixel16 = image->GetPixelImage(x,y);
-				sRGB16 newPixel16 = image->CalculateAmbientPixel(ambient,colorIndex,oldPixel16);
-				image->PutPixelImage(x,y,newPixel16);
-			}
-		}
-
-		if (!image->IsLowMemMode())
-		{
-			for (int x = 0; x <= width - progressive; x += progressive)
-			{
-				sRGB16 pixel = image->GetPixelAmbient(x, y);
-				for (int yy = 0; yy < progressive; yy++)
+				for(int sqX =0; sqX<progressive; sqX++)
 				{
-					for (int xx = 0; xx < progressive; xx++)
-					{
-						if (xx == 0 && yy == 0) continue;
-						image->PutPixelAmbient(x + xx, y + yy, pixel);
-					}
+					pixel.R = R;
+					pixel.G = G;
+					pixel.B = B;
+					image->PutPixelImage16(x + sqX, y + sqY, pixel);
 				}
 			}
 		}
+
 		double percentDone = (double) y / height * 100.0;
 		if(!quiet) printf("Rendering Screen Space Ambient Occlusion. Done %.2f%%       \r", percentDone);
 		fflush(stdout);
@@ -436,7 +441,7 @@ void PostRendering_SSAO(cImage *image, double persp, int quality, enumPerspectiv
 
 	int progressive = image->progressiveFactor;
 
-	if(image->IsLowMemMode()) progressive = 1;
+	//progressive = 1;
 
 	for (int i = 0; i < NR_THREADS; i++)
 	{
@@ -556,7 +561,7 @@ void DrawHistogram2(void)
 
 void DrawPalette(sRGB *palette)
 {
-	mainImage.SetPalette(palette);
+	memcpy(Interface_data.palette, palette, sizeof(sRGB)*256);
 
 	if (paletteViewCreated)
 	{
@@ -566,8 +571,19 @@ void DrawPalette(sRGB *palette)
 		for (int i = 0; i < 640; i++)
 		{
 			int number = (int) (i * 256.0 / colWidth + paletteOffset * 256.0);
-			sRGB color = mainImage.IndexToColour(number);
-			GdkColor gdk_color = { 0, color.R * 256, color.G * 256, color.B * 256 };
+			sRGB color = IndexToColour(number, palette);
+			GdkColor gdk_color;
+			gdk_color.red = color.R * 256;
+			gdk_color.green = color.G * 256;
+			gdk_color.blue = color.B * 256;
+			gdk_color.pixel = 0;
+			if(((int)(i + paletteOffset *10) % (int)colWidth) == colWidth / 2)
+			{
+				gdk_color.red = gdk_color.red > 32768 ? gdk_color.red - 5000 : gdk_color.red + 5000;
+				gdk_color.green = gdk_color.green > 32768 ? gdk_color.green - 5000 : gdk_color.green + 5000;
+				gdk_color.blue = gdk_color.blue > 32768 ? gdk_color.blue - 5000 : gdk_color.blue + 5000;
+				gdk_color.pixel = 0;
+			}
 			gdk_gc_set_rgb_fg_color(GC, &gdk_color);
 			gdk_draw_line(dareaPalette->window, GC, i, 0, i, 30);
 		}
