@@ -1,10 +1,13 @@
-/*
- * image.h
- *
- *  Created on: 2010-01-23
- *      Author: krzysztof marczak
- */
-
+/*********************************************************
+ /                   MANDELBULBER
+ / functions for image post-processing
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
 
 #ifndef IMAGE_H_
 #define IMAGE_H_
@@ -36,9 +39,8 @@ void NewPalette(sRGB *p, double nasycenie);
 void DrawHistogram(void);
 void DrawHistogram2(void);
 void PostRendering_SSAO(cImage *image, double persp, int quality, enumPerspectiveType perspectiveType, bool quiet);
-void PostRendering_DOF(cImage *image, double deep, double neutral, double persp);
+void PostRendering_DOF(cImage *image, double deep, double neutral);
 void DrawPalette(sRGB *palette);
-void StoreImage8(sComplexImage *image, sRGB8 *image8);
 void MakeStereoImage(cImage *left, cImage *right, guchar *stereoImage);
 void StereoPreview(cImage *temoraryImage, guchar *stereoImage);
 
