@@ -1,19 +1,27 @@
-/*
- * interface.h
- *
- *  Created on: 2010-01-23
- *      Author: krzysztof marczak
- */
+/*********************************************************
+ /                   MANDELBULBER
+ / definition of user interface and functions
+ / for parameters handling
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
 
 #ifndef INTERFACE_H_
 #define INTERFACE_H_
 
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include "fractparams.h"
 #include "cl_support.hpp"
+#include "callbacks.h"
+#include "shaders.h"
+#include "morph.hpp"
 
-#define MANDELBULBER_VERSION 1.15
-#define MANDELBULBER_VERSION_STR "1.15"
+#define MANDELBULBER_VERSION 1.211
+#define MANDELBULBER_VERSION_STR "1.21-1"
 
 #ifndef SHARED_DIR
 	#ifndef WIN32
@@ -23,6 +31,11 @@
 
 extern char *sharedDir;
 
+extern int x_mouse;
+extern int y_mouse;
+extern double last_z_mouse;
+extern double smooth_last_z_mouse;
+
 enum enumImageFormat
 {
 	imgFormatJPG = 0,
@@ -71,7 +84,7 @@ struct sInterface
 	GtkWidget *tab_label_fractal;
 	GtkWidget *tab_label_image;
 	GtkWidget *tab_label_animation;
-	GtkWidget *tab_label_posteffects;
+	GtkWidget *tab_label_shaders2;
 	GtkWidget *tab_label_lights;
 	GtkWidget *tab_label_IFS;
 	GtkWidget *tab_label_about;
@@ -81,18 +94,24 @@ struct sInterface
 	GtkWidget *tab_label_engine;
 	GtkWidget *tabsPrimitives;
 	GtkWidget *tab_label_primitivePlane;
+	GtkWidget *tab_label_primitiveWater;
 	GtkWidget *tab_label_primitiveBox;
+	GtkWidget *tab_label_primitiveBoxInv;
 	GtkWidget *tab_label_primitiveSphere;
+	GtkWidget *tab_label_primitiveSphereInv;
 	GtkWidget *tab_label_openCL;
 	GtkWidget *tabsNetRender;
 	GtkWidget *tab_label_server;
 	GtkWidget *tab_label_client;
+	GtkWidget *tabsOpenCL;
+	GtkWidget *tab_label_openclEngine;
+	GtkWidget *tab_label_openclCustom;
 
 	GtkWidget *tab_box_view;
 	GtkWidget *tab_box_fractal;
 	GtkWidget *tab_box_image;
 	GtkWidget *tab_box_animation;
-	GtkWidget *tab_box_posteffects;
+	GtkWidget *tab_box_shaders2;
 	GtkWidget *tab_box_lights;
 	GtkWidget *tab_box_IFS;
 	GtkWidget *tab_box_about;
@@ -101,11 +120,16 @@ struct sInterface
 	GtkWidget *tab_box_mandelbox;
 	GtkWidget *tab_box_engine;
 	GtkWidget *tab_box_primitivePlane;
+	GtkWidget *tab_box_primitiveWater;
 	GtkWidget *tab_box_primitiveBox;
+	GtkWidget *tab_box_primitiveBoxInv;
 	GtkWidget *tab_box_primitiveSphere;
+	GtkWidget *tab_box_primitiveSphereInv;
 	GtkWidget *tab_box_openCL;
 	GtkWidget *tab_box_server;
 	GtkWidget *tab_box_client;
+	GtkWidget *tab_box_openclEngine;
+	GtkWidget *tab_box_openclCustom;
 
 	GtkWidget *boxMain;
 	GtkWidget *boxButtons;
@@ -119,7 +143,6 @@ struct sInterface
 	GtkWidget *boxNavigation;
 	GtkWidget *boxNavigationButtons;
 	GtkWidget *boxNavigationZooming;
-	GtkWidget *boxFractal;
 	GtkWidget *boxFractalFormula;
 	GtkWidget *boxFractalFolding;
 	GtkWidget *boxFractalRayMarching;
@@ -236,6 +259,14 @@ struct sInterface
 	GtkWidget *boxOpenClEngineSettingsV;
 	GtkWidget *boxOpenClEngineSettingsH1;
 	GtkWidget *boxOpenClEngineSettingsH2;
+	GtkWidget *boxOpenClEngineSettingsH3;
+	GtkWidget *boxOpenClEngineSettingsH4;
+	GtkWidget *boxOpenClCustomV1;
+	GtkWidget *boxOpenClCustomH11;
+	GtkWidget *boxOpenClCustomH12;
+	GtkWidget *boxOpenClCustomH13;
+	GtkWidget *boxOpenClCustomV2;
+
 	GtkWidget *boxNetRenderServerV;
 	GtkWidget *boxNetRenderClientV;
 	GtkWidget *boxNetRenderServerH1;
@@ -243,6 +274,18 @@ struct sInterface
 	GtkWidget *boxFakeLightsV;
 	GtkWidget *boxFakeLightsH1;
 	GtkWidget *boxFakeLightsH2;
+	GtkWidget *boxImageAdjustmentsV;
+	GtkWidget *boxImageAdjustmentsH1;
+	GtkWidget *boxShadersSurfaceV;
+	GtkWidget *boxShadersSurfaceH1;
+	GtkWidget *boxShadersSurfaceH2;
+	GtkWidget *boxShadersSurfaceH3;
+	GtkWidget *boxShadersSurfaceH4;
+	GtkWidget *boxShadersVolumetricV;
+	GtkWidget *boxShadersVolumetricH1;
+	GtkWidget *boxShadersVolumetricH2;
+	GtkWidget *boxShadersVolumetricH3;
+	GtkWidget *boxShadersVolumetricH4;
 
 	GtkWidget *tableLimits;
 	GtkWidget *tableArrows;
@@ -250,10 +293,10 @@ struct sInterface
 	GtkWidget *tableIFSParams;
 	GtkWidget *tableHybridParams;
 	GtkWidget *tableMandelboxRotations;
+	GtkWidget *tableOpenCLCustom;
 
 	GtkWidget *frCoordinates;
 	GtkWidget *fr3Dnavigator;
-	GtkWidget *frFractal;
 	GtkWidget *frLimits;
 	GtkWidget *frImage;
 	GtkWidget *frEffects;
@@ -299,8 +342,13 @@ struct sInterface
 	GtkWidget *frOpenClSettings;
 	GtkWidget *frOpenClInformation;
 	GtkWidget *frOpenClEngineSettings;
+	GtkWidget *frOpenClCustomSelection;
+	GtkWidget *frOpenClCustomParams;
 	GtkWidget *frNetRender;
 	GtkWidget *frFakeLights;
+	GtkWidget *frImageAdjustments;
+	GtkWidget *frShadersSurface;
+	GtkWidget *frShadersVolumetric;
 
 	GtkWidget *hSeparator1;
 	GtkWidget *hSeparator2;
@@ -318,7 +366,7 @@ struct sInterface
 	GtkWidget *buColorFog1;
 	GtkWidget *buColorFog2;
 	GtkWidget *buColorFog3;
-	GtkWidget *buApplyBrighness;
+	GtkWidget *buApplyImageAdjustments;
 	GtkWidget *buSaveImage;
 	GtkWidget *buSavePNG;
 	GtkWidget *buSavePNG16;
@@ -381,6 +429,12 @@ struct sInterface
 	GtkWidget *buAutoFog;
 	GtkWidget *buMeasureActivation;
 	GtkWidget *buSaveAllImageLayers;
+	GtkWidget *buOpenCLNewFormula;
+	GtkWidget *buOpenCLDeleteFormula;
+	GtkWidget *buOpenCLEditFormula;
+	GtkWidget *buOpenCLEditFormulaInit;
+	GtkWidget *buOpenCLRecompile;
+	GtkWidget *buConvertPathToKeyframes;
 
 	GtkWidget *edit_va;
 	GtkWidget *edit_vb;
@@ -414,6 +468,7 @@ struct sInterface
 	GtkWidget *edit_ambient;
 	GtkWidget *edit_brightness;
 	GtkWidget *edit_gamma;
+	GtkWidget *edit_contrast;
 	GtkWidget *edit_specular;
 	GtkWidget *edit_reflect;
 	GtkWidget *edit_AmbientOcclusionQuality;
@@ -504,8 +559,6 @@ struct sInterface
 	GtkWidget *edit_FoldingIntPowZFactor;
 	GtkWidget *edit_FoldingIntPowFoldingFactor;
 	GtkWidget *edit_roughness;
-	GtkWidget *edit_volumetricLightQuality;
-	GtkWidget *edit_volumetricLightIntensity;
 	GtkWidget *edit_volumetricLightMainIntensity;
 	GtkWidget *edit_volumetricLightAux1Intensity;
 	GtkWidget *edit_volumetricLightAux2Intensity;
@@ -529,36 +582,46 @@ struct sInterface
 	GtkWidget *edit_primitivePlaneNormalX;
 	GtkWidget *edit_primitivePlaneNormalY;
 	GtkWidget *edit_primitivePlaneNormalZ;
+	GtkWidget *edit_primitivePlaneReflect;
 	GtkWidget *edit_primitiveBoxCentreX;
 	GtkWidget *edit_primitiveBoxCentreY;
 	GtkWidget *edit_primitiveBoxCentreZ;
 	GtkWidget *edit_primitiveBoxSizeX;
 	GtkWidget *edit_primitiveBoxSizeY;
 	GtkWidget *edit_primitiveBoxSizeZ;
+	GtkWidget *edit_primitiveBoxReflect;
 	GtkWidget *edit_primitiveInvertedBoxCentreX;
 	GtkWidget *edit_primitiveInvertedBoxCentreY;
 	GtkWidget *edit_primitiveInvertedBoxCentreZ;
 	GtkWidget *edit_primitiveInvertedBoxSizeX;
 	GtkWidget *edit_primitiveInvertedBoxSizeY;
 	GtkWidget *edit_primitiveInvertedBoxSizeZ;
+	GtkWidget *edit_primitiveInvertedBoxReflect;
 	GtkWidget *edit_primitiveSphereCentreX;
 	GtkWidget *edit_primitiveSphereCentreY;
 	GtkWidget *edit_primitiveSphereCentreZ;
 	GtkWidget *edit_primitiveSphereRadius;
+	GtkWidget *edit_primitiveSphereReflect;
 	GtkWidget *edit_primitiveInvertedSphereCentreX;
 	GtkWidget *edit_primitiveInvertedSphereCentreY;
 	GtkWidget *edit_primitiveInvertedSphereCentreZ;
 	GtkWidget *edit_primitiveInvertedSphereRadius;
+	GtkWidget *edit_primitiveInvertedSphereReflect;
 	GtkWidget *edit_primitiveWaterHeight;
 	GtkWidget *edit_primitiveWaterAmplitude;
 	GtkWidget *edit_primitiveWaterLength;
 	GtkWidget *edit_primitiveWaterRotation;
 	GtkWidget *edit_primitiveWaterIterations;
+	GtkWidget *edit_primitiveWaterReflect;
 	GtkWidget *edit_measureX;
 	GtkWidget *edit_measureY;
 	GtkWidget *edit_measureZ;
 	GtkWidget *edit_tiles;
 	GtkWidget *edit_OpenCLPixelsPerJob;
+	GtkWidget *edit_OpenCLProcessingCycleTime;
+	GtkWidget *edit_OpenCLMaxMem;
+	GtkWidget *edit_OpenCLTextEditor;
+	GtkWidget *edit_OpenCLDeltaDEStep;
 	GtkWidget *edit_iterFogOpacityTrim;
 	GtkWidget *edit_iterFogOpacity;
 	GtkWidget *edit_netRenderServerPort;
@@ -572,12 +635,14 @@ struct sInterface
 	GtkWidget *edit_fakeLightsVisibilitySize;
 	GtkWidget *edit_fakeLightsMinIter;
 	GtkWidget *edit_fakeLightsMaxIter;
+	GtkWidget *edit_shadowConeAngle;
+	GtkWidget *edit_customParameters[15];
+	GtkWidget *edit_primitiveWaterAnimSpeed;
 
 	GtkWidget *label_animationFrame;
 	GtkWidget *label_animationSpeed;
 	GtkWidget *label_animationDistance;
 	GtkWidget *label_fog_visibility;
-	GtkWidget *label_fog_visibility_front;
 	GtkWidget *label_SSAO_quality;
 	GtkWidget *label_DOF_focus;
 	GtkWidget *label_DOF_radius;
@@ -610,6 +675,8 @@ struct sInterface
 	GtkWidget *label_OpenClStatus;
 	GtkWidget *label_serverStatus;
 	GtkWidget *label_clientStatus;
+	GtkWidget *label_sliderFog;
+	GtkWidget *label_sliderDOF;
 
 	GtkWidget *comboFractType;
 	GtkWidget *comboImageFormat;
@@ -619,6 +686,12 @@ struct sInterface
 	GtkWidget *comboImageProportion;
 	GtkWidget *comboGeneralizedFoldBoxType;
 	GtkWidget *comboOpenCLEngine;
+	GtkWidget *comboOpenCLDeviceIndex;
+	GtkWidget *comboOpenCLPlatformIndex;
+	GtkWidget *comboOpenCLGPUCPU;
+	GtkWidget *comboOpenCLCustomFormulas;
+	GtkWidget *comboOpenCLDEMode;
+	GtkWidget *comboOpenCLDOFMode;
 
 	GtkWidget *progressBar;
 
@@ -656,7 +729,6 @@ struct sInterface
 	GtkWidget *checkStereoEnabled;
 	GtkWidget *checkMandelboxRotationsEnable;
 	GtkWidget *checkInteriorMode;
-	GtkWidget *checkDECorrectionMode;
 	GtkWidget *checkDELinearMode;
 	GtkWidget *checkConstantDEThreshold;
 	GtkWidget *checkVolumetricLightMainEnabled;
@@ -673,12 +745,14 @@ struct sInterface
 	GtkWidget *checkPrimitiveInvertedSphereEnabled;
 	GtkWidget *checkPrimitiveWaterEnabled;
 	GtkWidget *checkOpenClEnable;
+	GtkWidget *checkOpenClCustomEnable;
 	GtkWidget *checkIterFogEnable;
 	GtkWidget *checkNetRenderServerEnable;
 	GtkWidget *checkNetRenderServerScan;
 	GtkWidget *checkNetRenderClientEnable;
 	GtkWidget *checkPrimitiveOnlyPlane;
-	GtkWidget *checkFakeLightsEnabled;;
+	GtkWidget *checkFakeLightsEnabled;
+	GtkWidget *checkHDR;
 
 	GtkWidget *colorSelectionGlow1;
 	GtkWidget *colorSelectionGlow2;
@@ -695,14 +769,12 @@ struct sInterface
 	GtkWidget *pixmap_move_right;
 
 	GtkWidget *sliderFogDepth;
-	GtkWidget *sliderFogDepthFront;
 	GtkWidget *sliderSSAOQuality;
 	GtkWidget *sliderDOFFocus;
 	GtkWidget *sliderDOFRadius;
 	GtkWidget *sliderPaletteOffset;
 
 	GtkObject *adjustmentFogDepth;
-	GtkObject *adjustmentFogDepthFront;
 	GtkObject *adjustmentSSAOQuality;
 	GtkObject *adjustmentDOFFocus;
 	GtkObject *adjustmentDOFRadius;
@@ -791,6 +863,11 @@ struct sInterface_data
 	char file_lightmap[1000];
 	char file_path[1000];
 	char file_keyframes[1000];
+
+	int franeNo;
+	int tileCount;
+
+	sRGB palette[256];
 };
 
 struct sNoGUIdata
@@ -799,7 +876,6 @@ struct sNoGUIdata
 	bool animMode;
 	bool playMode;
 	bool keyframeMode;
-	bool lowMemMode;
 	bool netrenderMode;
 	int startFrame;
 	int endFrame;
@@ -809,6 +885,30 @@ struct sNoGUIdata
 	sParamRender fractparams;
 	std::vector<const char *> overrideStrings;
 };
+
+struct sAppSettings
+{
+#ifdef CLSUPPORT
+	bool oclUseCPU;
+	int oclDeviceIndex;
+	int oclPlatformIndex;
+	int oclEngineSelection;
+	double oclCycleTime;
+	double oclMemoryLimit;
+	std::string oclTextEditor;
+#endif
+	bool absoluteMovementModeEnabled;
+	bool zoomByMouseClickEnabled;
+	bool goCloseToSurfaceEnabled;
+	double cameraMoveStepRelative;
+	double cameraMoveStepAbsolute;
+	double rotationStep;
+	double mouseCloseUpRatio;
+  std::string netRenderClientPort;
+	std::string netRenderClientIP;
+	std::string netRenderServerPort;
+};
+
 //Global variables
 
 extern sMainWindow renderWindow;
@@ -854,10 +954,12 @@ void CheckPrameters(sParamRender *params);
 sRGB GdkColor2sRGB(GdkColor color);
 GdkColor sRGB2GdkColor(sRGB color);
 sRGB sRGBDiv256(sRGB color);
+void ReadInterfaceAppSettings(sAppSettings *appParams);
+void WriteInterfaceAppSettings(sAppSettings *appParams);
 #ifdef CLSUPPORT
-void Params2Cl(const sParamRender *params, sClParams *clParams, sClFractal *clFractal, sClInBuff *clInBuff);
+void Params2Cl(const sParamRender *params, sClInBuff *clInBuff, sClInConstants *clConstantsBuff);
 matrix33 RotMatrix2matrix33(CRotationMatrix rot);
-cl_float4 CVector2float4(CVector3 vect);
-cl_float4 sRGB2float4(sRGB colour, double factor);
+cl_float4 CVector2float3(CVector3 vect);
+cl_float4 sRGB2float3(sRGB colour, double factor);
 #endif
 #endif /* INTERFACE_H_ */
