@@ -1,9 +1,13 @@
-/*
- * netrender.cpp
- *
- *  Created on: 28-09-2012
- *      Author: krzysztof marczak
- */
+/*********************************************************
+ /                   MANDELBULBER
+ / class for rendering with several network clients
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
 
 #include "netrender.hpp"
 #include <stdio.h>
@@ -55,7 +59,7 @@ bool CNetRender::SetServer(char *portNo, char *statusOut)
 {
   memset(&host_info, 0, sizeof host_info);
 
-  host_info.ai_family = AF_UNSPEC;     // IP version not specified. Can be both.
+  host_info.ai_family = AF_INET;     // IP version not specified. Can be both.
   host_info.ai_socktype = SOCK_STREAM; // Use SOCK_STREAM for TCP or SOCK_DGRAM for UDP.
   host_info.ai_flags = AI_PASSIVE;     // IP Wildcard
 
@@ -124,7 +128,7 @@ bool CNetRender::SetClient(char *portNo, char*name, char *statusOut)
 {
 	memset(&host_info, 0, sizeof host_info);
 
-	host_info.ai_family = AF_UNSPEC;     // IP version not specified. Can be both.
+	host_info.ai_family = AF_INET;     // IP version not specified. Can be both.
 	host_info.ai_socktype = SOCK_STREAM; // Use SOCK_STREAM for TCP or SOCK_DGRAM for UDP.
 
 	status = getaddrinfo(name, portNo, &host_info, &host_info_list);
@@ -173,7 +177,7 @@ bool CNetRender::SetClient(char *portNo, char*name, char *statusOut)
 		if(!strcmp("ver", command))
 		{
 	  	size_t len = sizeof(version);
-	  	sendDataToServer(&version, len, "VER");
+	  	sendDataToServer(&version, len, (char*)"VER");
 		}
 
   	//checking aswer regarding version
@@ -186,14 +190,14 @@ bool CNetRender::SetClient(char *portNo, char*name, char *statusOut)
   		recvd_bytes = receiveDataFromServer(command);
   		if(!strcmp("cpu", command))
   		{
-  			sendDataToServer(&noOfCPUs, sizeof(noOfCPUs), "CPU");
+  			sendDataToServer(&noOfCPUs, sizeof(noOfCPUs), (char*)"CPU");
   		}
 
     	return true;
 		}
 		else if(!strcmp("bad", command))
 		{
-    	int serverVersion;
+    	int serverVersion = 0;
     	if(recvd_bytes == sizeof(int)) GetData(&serverVersion);
 			printf("Client version refused. Server version is %f\n", serverVersion/1000.0);
   		isClient = false;
@@ -312,8 +316,8 @@ bool CNetRender::WaitForClient(char *statusOut)
 		std::cout << stat;
 
 		//checking client version
-		int clientVersion;
-		sendDataToClient(NULL, 0, "ver", clientIndex-1, 0);
+		int clientVersion = 0;
+		sendDataToClient(NULL, 0, (char*)"ver", clientIndex-1, 0);
 		char command[4];
 		size_t recvd_bytes = receiveDataFromClient(command, clientIndex-1, 0);
 		if(!strcmp("VER", command) && recvd_bytes == sizeof(int))
@@ -326,10 +330,10 @@ bool CNetRender::WaitForClient(char *statusOut)
 			printf("Client version is correct\n");
 
 			//sending answer
-			sendDataToClient(NULL, 0, "ok.", clientIndex-1, 0);
+			sendDataToClient(NULL, 0, (char*)"ok.", clientIndex-1, 0);
 
 			//ask for number of CPUs
-			sendDataToClient(NULL, 0, "cpu", clientIndex-1, 0);
+			sendDataToClient(NULL, 0, (char*)"cpu", clientIndex-1, 0);
 			recvd_bytes = receiveDataFromClient(command, clientIndex-1, 0);
 			if(!strcmp("CPU", command) && recvd_bytes == sizeof(int))
 			{
@@ -346,7 +350,7 @@ bool CNetRender::WaitForClient(char *statusOut)
 			printf("Wrong client version\n");
 
 			//sending answer
-			sendDataToClient(&version, sizeof(version), "bad", clientIndex-1, 0);
+			sendDataToClient(&version, sizeof(version), (char*)"bad", clientIndex-1, 0);
 
 			//deleting client
 			clients.erase(clients.end()-1);
