@@ -1,9 +1,13 @@
-/*
- * primitives.hpp
- *
- *  Created on: 15-08-2011
- *      Author: krzysztof
- */
+/*********************************************************
+ /                   MANDELBULBER
+ / functions for calculating shapes of primitive objects
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
 
 #ifndef PRIMITIVES_H_
 #define PRIMITIVES_H_
