@@ -1,14 +1,20 @@
-/*
- * settings.h
- *
- *  Created on: 2010-01-24
- *      Author: krzysztof marczak
- */
+/*********************************************************
+ /                   MANDELBULBER
+ / functions loading and saving parameters
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ / many improvements done by Rayan Hitchman
+ ********************************************************/
 
 #ifndef SETTINGS_H_
 #define SETTINGS_H_
 
 #include "Render3D.h"
+#include "interface.h"
 
 extern char data_directory[1000];
 
@@ -25,4 +31,7 @@ void MorphToIFS(double *IFSdouble, sFractal &fractal);
 void MakePaletteString(const sRGB *palette, char *paletteString);
 void GetPaletteFromString(sRGB *palette, const char *paletteString);
 void KeepOtherSettings(sParamRender *params);
+void SaveAppSettings(const char *filename, const sAppSettings& appParams);
+bool LoadAppSettings(char *filename, sAppSettings &appParams);
+
 #endif /* SETTINGS_H_ */
