@@ -1,16 +1,23 @@
-/*
- * texture.cpp
- *
- *  Created on: 2010-02-07
- *      Author: krzysztof
- */
+/*********************************************************
+ /                   MANDELBULBER
+ / class for bitmap textures
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
+
 #include <stdio.h>
+#include <cstring>
 #include "texture.hpp"
 #include "files.h"
 
 //constructor
 cTexture::cTexture(const char *filename)
 {
+	bitmap = NULL;
 	if (CheckJPEGsize(filename, &width, &height))
 	{
 
@@ -69,7 +76,7 @@ cTexture::cTexture(void)
 //destructor
 cTexture::~cTexture(void)
 {
-	delete[] bitmap;
+	if (bitmap) delete[] bitmap;
 }
 
 //read pixel
