@@ -1,9 +1,13 @@
-/*
- * timeline.hpp
- *
- *  Created on: 2010-09-20
- *      Author: krzysztof
- */
+/*********************************************************
+ /                   MANDELBULBER
+ / class for animation timeline
+ /
+ /
+ / author: Krzysztof Marczak
+ / contact: buddhi1980@gmail.com
+ / licence: GNU GPL v3.0
+ /
+ ********************************************************/
 
 #ifndef TIMELINE_HPP_
 #define TIMELINE_HPP_
@@ -11,7 +15,10 @@
 #include "database.hpp"
 #include "cimage.hpp"
 #include "smartptr.h"
-
+#include "files.h"
+#include "interface.h"
+#include "settings.h"
+#include "callbacks.h"
 
 gboolean thumbnail_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
 void PressedKeyframeThumbnail(GtkWidget *widget, GdkEventButton *event);
@@ -40,12 +47,16 @@ public:
 	void Resize(int newsize);
 	void Reset(void);
 	void Refresh(void);
+	void UpdateGlobalMorph(void);
+	void GetFrameParamsInterpolated(int index, int framesPerKey, sParamRenderD *params);
+	int GetKeyframeCount(void) {return keyframeCount;}
 	bool isOpened;
 
 private:
 	smart_ptr<cDatabase> database;
 	int keyframeCount;
 	bool isCreated;
+	CMorph *morphParamRenderGlobal;
 };
 
 extern smart_ptr<cTimeline> timeline;
